// Copyright (c) 2015  John Abbott
// This file is part of the CoCoALib suite of examples.
// You are free to use any part of this example in your own programs.

#include "CoCoA/library.H"

#include <csignal>
using namespace std;

//----------------------------------------------------------------------
const string ShortDescription =
  "This short example shows how to specify a global interrupt flag,  \n"
  "and how CheckForInterrupt can be used.                            \n";

const string LongDescription =
  "This short example shows how to specify a global interrupt flag,  \n"
  "and how CheckForInterrupt can be used.                            \n";
//----------------------------------------------------------------------

namespace CoCoA
{
  // Global interrupt flag
  volatile std::sig_atomic_t GlobalInterruptFlag = 0;

  void SetInterruptFlag(int)
  {
    GlobalInterruptFlag = true;
  }

  ///////////////////////////////////////////////////////
  // A long computation which checks occasionally for interrupts
  void LongComputation()
  {
    BigRat prod(1,1);
    const int MaxPrime = 700000; // so it takes about 4s on my computer
    for (int p=2; p < MaxPrime; p = NextPrime(p))
    {
      CheckForInterrupt("LongComputation"); // arg gives context info
      prod *= BigRat(p-1,p);
    }
    cout << "Product of (1-1/p) for primes p up to " << MaxPrime
         << " is about " << FloatStr(prod) << endl;
  }


  void program()
  {
    cout << ShortDescription << endl;
    GlobalManager CoCoAFoundations;
    // Tell GlobalManager which flag is to be monitored.
    CoCoAFoundations.myNewInterruptFlagPtr(&GlobalInterruptFlag);

    // Tell system to call SetInterruptFlag when SIGINT arrives
    struct sigaction NewAction;
    NewAction.sa_handler = SetInterruptFlag;
    sigemptyset(&NewAction.sa_mask);
    NewAction.sa_flags = 0;
    sigaction(SIGINT, &NewAction, 0/*NULL*/);

    // Now do a long computation which checks for interrupts...
    try
    {
      LongComputation();
    }
    catch (const InterruptReceived&)
    {
      cout << "\n***LongComputation was interrupted***\n" << endl;
    }

  }

} // end of namespace CoCoA

//----------------------------------------------------------------------
// Use main() to handle any uncaught exceptions and warn the user about them.
int main()
{
  try
  {
    CoCoA::program();
    return 0;
  }
  catch (const CoCoA::ErrorInfo& err)
  {
    cerr << "***ERROR***  UNCAUGHT CoCoA error";
    ANNOUNCE(cerr, err);
  }
  catch (const std::exception& exc)
  {
    cerr << "***ERROR***  UNCAUGHT std::exception: " << exc.what() << endl;
  }
  catch(...)
  {
    cerr << "***ERROR***  UNCAUGHT UNKNOWN EXCEPTION" << endl;
  }

  CoCoA::BuildInfo::PrintAll(cerr);
  return 1;
}

//----------------------------------------------------------------------
// RCS header/log in the next few lines
// $Header: /Volumes/Home_1/cocoa/cvs-repository/CoCoALib-0.99/examples/ex-interrupt1.C,v 1.4 2015/11/04 10:12:41 abbott Exp $
// $Log: ex-interrupt1.C,v $
// Revision 1.4  2015/11/04 10:12:41  abbott
// Summary: Made it faster
//
// Revision 1.3  2015/06/26 14:55:20  abbott
// Summary: Improved readability
// Author: JAA
//
// Revision 1.2  2015/06/25 14:49:51  abbott
// Summary: Added string arg to CheckForInterrupt
// Author: JAA
//
// Revision 1.1  2015/05/20 16:52:43  abbott
// Summary: New example illustrating checking and handling interrupts
// Author: JAA
//
//
